@@ -14,6 +14,11 @@ use warnings;
     }
     my $p_peaks = Algorithm::Evolutionary::Fitness::P_Peaks->new( $number_of_bits, @weights ); #Number of peaks = scalar  @weights
 
+    # Or use an alternative ctor
+    my $descriptor = { number_of_peaks => 100,
+                       weight => 0.99 };
+    my $p_peaks = Algorithm::Evolutionary::Fitness::P_Peaks->new( $number_of_bits, $descriptor)
+
 =head1 DESCRIPTION
 
     wP-Peaks fitness function, weighted version of the P-Peaks fitness function, which has now a single peak
@@ -24,30 +29,43 @@ use warnings;
 
 package Algorithm::Evolutionary::Fitness::wP_Peaks;
 
-our $VERSION = ( '$Revision: 1.2 $ ' =~ /(\d+\.\d+)/ ) ;
+our $VERSION = ( '$Revision: 2.1 $ ' =~ /(\d+\.\d+)/ ) ;
 
 use String::Random;
 use Algorithm::Evolutionary::Utils qw(hamming);
 
-use base qw(Algorithm::Evolutionary::Fitness::Base);
+use base qw(Algorithm::Evolutionary::Fitness::String);
 
-=head2 new( $number_of_bits, $ref_to_weights_hash )
+=head2 new( $number_of_bits, @weights_array )
 
-    Creates a new instance of the problem, with the said number of bits and peaks
+or new( $number_of_bits, $hash_with_number_of_peaks_and_weight )
+
+Creates a new instance of the problem, with the said number of bits and peaks
 
 =cut 
 
 sub new {
   my $class = shift;
-  my ( $bits, @weights ) = @_;
+  my ( $bits ) = shift;
+
+  my @weights;
+  if ( ref $_[0] eq 'HASH' ) {
+      push @weights, 1;
+      for (my $i = 0; $i < $_[0]->{'number_of_peaks'}-1; $i ++ ) {
+	  push @weights, $_[0]->{'weight'};
+      } 
+  } else {
+      @weights = @_;
+  }
+  my $self = $class->SUPER::new();
 
   #Generate peaks
   my $generator = new String::Random;
   my @peaks;
   my $regex = "\[01\]{$bits}";
-  my $self = { bits => $bits,
-	       generator => $generator,
-	       regex => $regex };
+  for my $s qw( bits generator regex ) {
+      eval "\$self->{'$s'} = \$$s";
+  }
   while ( @weights ) {
       my $this_w = shift @weights;
       push( @peaks, [$this_w, $generator->randregex($regex)]);
@@ -69,16 +87,9 @@ sub random_string {
     return $self->{'generator'}->randregex($self->{'regex'});
 }
 
-=head2 _apply
-
-Applies the instantiated problem to a chromosome
-
-=cut
-
-sub _apply {
+sub _really_apply {
     my $self = shift;
-    my $individual = shift;
-    return  $self->p_peaks( $individual->{_str})/$self->{'bits'};
+    return  $self->p_peaks( @_ )/$self->{'bits'};
 }
 
 =head2 p_peaks
@@ -87,43 +98,32 @@ Applies the instantiated problem to a string
 
 =cut
 
-our %cache;
-
 sub p_peaks {
     my $self = shift;
     my @peaks = @{$self->{'peaks'}};
     my $string = shift;
-    if ( $cache{$string} ) {
-	return $cache{$string};
+    my $cache = $self->{'_cache'};
+    
+    if ( $cache->{$string} ) {
+	return $cache->{$string};
     }
     my $bits = $self->{'bits'};
     my @distances = sort {$b <=> $a} map($bits*$_->[0]- hamming( $string, $_->[1]), @peaks);
-    $cache{$string} = $distances[0];
-    return $cache{$string};
+    $cache->{$string} = $distances[0];
+    return $cache->{$string};
 
 }
 
-=head2 cached_evals
-
-Returns the number of keys in the evaluation cache
-
-=cut
-
-sub cached_evals {
-    return scalar keys %cache;
-}
-
-
 
 =head1 Copyright
   
   This file is released under the GPL. See the LICENSE file included in this distribution,
   or go to http://www.fsf.org/licenses/gpl.txt
 
-  CVS Info: $Date: 2008/06/22 07:51:21 $ 
-  $Header: /cvsroot/opeal/Algorithm-Evolutionary/lib/Algorithm/Evolutionary/Fitness/wP_Peaks.pm,v 1.2 2008/06/22 07:51:21 jmerelo Exp $ 
+  CVS Info: $Date: 2009/02/04 20:43:14 $ 
+  $Header: /cvsroot/opeal/Algorithm-Evolutionary/lib/Algorithm/Evolutionary/Fitness/wP_Peaks.pm,v 2.1 2009/02/04 20:43:14 jmerelo Exp $ 
   $Author: jmerelo $ 
-  $Revision: 1.2 $
+  $Revision: 2.1 $
   $Name $
 
 =cut
