@@ -36,24 +36,29 @@ population.
 
 package Algorithm::Evolutionary::Op::CanonicalGA;
 
-our $VERSION = ( '$Revision: 1.2 $ ' =~ /(\d+\.\d+)/ ) ;
+use lib qw(../../..);
+
+our $VERSION = ( '$Revision: 2.1 $ ' =~ /(\d+\.\d+)/ ) ;
 
 use Carp;
+use Clone::Fast qw(clone);
+
 use Algorithm::Evolutionary::Wheel;
 use Algorithm::Evolutionary::Op::Bitflip;
 use Algorithm::Evolutionary::Op::QuadXOver;
 
-use Algorithm::Evolutionary::Op::Easy;
-our @ISA = qw(Algorithm::Evolutionary::Op::Easy);
+use base 'Algorithm::Evolutionary::Op::Easy';
 
 # Class-wide constants
 our $APPLIESTO =  'ARRAY';
 our $ARITY = 1;
 
-=head2 new
+=head2 new( $fitness[, $selection_rate][,$operators_ref_to_array] )
 
 Creates an algorithm, with the usual operators. Includes a default mutation
-and crossover, in case they are not passed as parameters
+and crossover, in case they are not passed as parameters. The first
+    element in the array ref should be an unary, and the second a
+    binary operator.
 
 =cut
 
@@ -76,12 +81,12 @@ sub new {
 
 }
 
-=head2 apply
-
-Applies the algorithm to the population; checks that it receives a
-ref-to-array as input, croaks if it does not. Returns a sorted,
-culled, evaluated population for next generation.
+=head2 apply( $population) 
 
+Applies a single generation of the algorithm to the population; checks
+that it receives a ref-to-array as input, croaks if it does
+not. Returns a sorted, culled, evaluated population for next
+generation.
 
 =cut
 
@@ -92,45 +97,53 @@ sub apply ($) {
 
   my $eval = $self->{_eval};
   for ( @$pop ) {
-    my $fitness;  #Evaluates only those that have no fitness
     if ( !defined ($_->Fitness() ) ) {
 	$_->evaluate( $eval );
     }
   }
 
   my @newPop;
-  my @rates;
-  for ( @$pop ) {
-    push @rates, $_->{_fitness};
-  }
-  my $popWheel=new Algorithm::Evolutionary::Wheel @rates;
+  my @rates = map( $_->Fitness(), @$pop );
+
+  #Creates a roulette wheel from the op priorities. Theoretically,
+  #they might have changed 
+  my $popWheel= new Algorithm::Evolutionary::Wheel @rates;
   my $popSize = scalar @$pop;
   my @ops = @{$self->{_ops}};
   for ( my $i = 0; $i < $popSize*(1-$self->{_selrate})/2; $i ++ ) {
-    my $clone1 = $pop->[$popWheel->spin()]->clone();
-    my $clone2 = $pop->[$popWheel->spin()]->clone();
-    $clone1 = $ops[0]->apply($clone1 ); # This should be a mutation-like op
-    $clone2 = $ops[0]->apply( $clone2 );
-    $ops[1]->apply( $clone1, $clone2 ); #This should be a
+#    my $clone1 = $pop->[$popWheel->spin()]->clone();
+#    my $clone2 = $pop->[$popWheel->spin()]->clone();
+#       my $clone1 = clone($pop->[$popWheel->spin()]);
+#       my $clone2 = clone($pop->[$popWheel->spin()]);
+      my $clone1 = $ops[0]->apply( $pop->[$popWheel->spin()] ); # This should be a mutation-like op
+      my $clone2 = $ops[0]->apply( $pop->[$popWheel->spin()] );
+#       $clone1 = $ops[0]->apply( $clone1 ); # This should be a mutation-like op
+#       $clone2 = $ops[0]->apply( $clone2 ); # This should be a mutation-like op
+      $ops[1]->apply( $clone1, $clone2 ); #This should be a
                                           #crossover-like op
-    $clone1->Fitness( $eval->( $clone1 ));
-    $clone2->Fitness( $eval->( $clone2 ));
-    push @newPop, $clone1, $clone2;
+      $clone1->evaluate( $eval );
+      $clone2->evaluate( $eval );
+      push @newPop, $clone1, $clone2;
   }
   #Re-sort
   @{$pop}[$popSize*$self->{_selrate}..$popSize-1] =  @newPop;
   @$pop = sort { $b->{_fitness} <=> $a->{_fitness} } @$pop;
 }
 
+
+=head1 SEE ALSO
+
+L<Algorithm::Evolutionary::Op::Easy>.
+
 =head1 Copyright
   
   This file is released under the GPL. See the LICENSE file included in this distribution,
   or go to http://www.fsf.org/licenses/gpl.txt
 
-  CVS Info: $Date: 2008/06/23 11:27:10 $ 
-  $Header: /cvsroot/opeal/Algorithm-Evolutionary/lib/Algorithm/Evolutionary/Op/CanonicalGA.pm,v 1.2 2008/06/23 11:27:10 jmerelo Exp $ 
+  CVS Info: $Date: 2009/02/04 20:43:14 $ 
+  $Header: /cvsroot/opeal/Algorithm-Evolutionary/lib/Algorithm/Evolutionary/Op/CanonicalGA.pm,v 2.1 2009/02/04 20:43:14 jmerelo Exp $ 
   $Author: jmerelo $ 
-  $Revision: 1.2 $
+  $Revision: 2.1 $
   $Name $
 
 =cut
