@@ -1,6 +1,8 @@
-use strict;
+use strict; #-*-cperl-*-
 use warnings;
 
+use lib qw( ../../.. );
+
 =head1 NAME
 
 Algorithm::Evolutionary::Op::Easy - evolutionary algorithm, single generation, with 
@@ -21,7 +23,7 @@ Algorithm::Evolutionary::Op::Easy - evolutionary algorithm, single generation, w
   }
 
   #Define a default algorithm with predefined evaluation function,
-  #Mutation and crossover. Default selection rate ls 4
+  #Mutation and crossover. Default selection rate is 0.4
   my $algo = new Algorithm::Evolutionary::Op::Easy( $eval ); 
 
   #Define an easy single-generation algorithm with predefined mutation and crossover
@@ -31,7 +33,7 @@ Algorithm::Evolutionary::Op::Easy - evolutionary algorithm, single generation, w
 
 =head1 Base Class
 
-L<Algorithm::Evolutionary::Op::Base|Algorithm::Evolutionary::Op::Base>
+L<Algorithm::Evolutionary::Op::Base>
 
 =cut
 
@@ -48,19 +50,19 @@ iteration of the algorithm to the population it takes as input
 
 package Algorithm::Evolutionary::Op::Easy;
 
-our $VERSION = ( '$Revision: 1.5 $ ' =~ /(\d+\.\d+)/ ) ;
+our $VERSION = ( '$Revision: 2.1 $ ' =~ / (\d+\.\d+)/ ) ;
 
 use Carp;
+use Clone::Fast qw(clone);
+
 use Algorithm::Evolutionary::Wheel;
 use Algorithm::Evolutionary::Op::Bitflip;
 use Algorithm::Evolutionary::Op::Crossover;
 
-use Algorithm::Evolutionary::Op::Base;
-our @ISA = qw(Algorithm::Evolutionary::Op::Base);
+use base 'Algorithm::Evolutionary::Op::Base';
 
 # Class-wide constants
 our $APPLIESTO =  'ARRAY';
-our $ARITY = 1;
 
 =head2 new( $eval_func, [$operators_arrayref] )
 
@@ -127,7 +129,6 @@ culled, evaluated population for next generation.
 sub apply ($) {
   my $self = shift;
   my $pop = shift || croak "No population here";
-  croak "Incorrect type ".(ref $pop) if  ref( $pop ) ne $APPLIESTO;
 
   #Evaluate
   my $eval = $self->{_eval};
@@ -143,23 +144,14 @@ sub apply ($) {
 
   #Sort
   my @popsort = sort { $b->{_fitness} <=> $a->{_fitness}; }
-					  @popEval ;
+    @popEval ;
 
   #Cull
   my $pringaos = int(($#popsort+1)*$self->{_selrate}); #+1 gives you size
   splice @popsort, $#popsort - $pringaos, $pringaos;
  
   #Reproduce
-  my $totRate = 0;
-  my @rates;
-  for ( @ops ) {
-      if ( $_->{'rate'} ) {
-	  push( @rates, $_->{'rate'});
-      } else {
-	  croak 'Operator has no rate!n\n';
-      }
-
-  }
+  my @rates = map( $_->{'rate'}, @ops );
   my $opWheel = new Algorithm::Evolutionary::Wheel @rates;
 
   #Generate offpring;
@@ -168,24 +160,12 @@ sub apply ($) {
       my @offspring;
       my $selectedOp = $ops[ $opWheel->spin()];
       for ( my $j = 0; $j < $selectedOp->arity(); $j ++ ) {
-	  my $chosen = $popsort[ rand( $originalSize )];
-	  push( @offspring, $chosen->clone() );
+	  my $chosen = $popsort[ int ( rand( $originalSize ) )];
+	  push( @offspring, $chosen ); #No need to clone, it's not changed in ops
       }
       my $mutante = $selectedOp->apply( @offspring );
       push( @popsort, $mutante );
   }
-#  for ( @ops ) {
-#	my $relRate = $_->{rate} / $totRate;
-#	for ( my $i = 0; $i < $pringaos*$relRate; $i++ ) {
-#	  my @offspring;
-#	  for ( my $j = 0; $j < $_->arity(); $j ++ ) {
-#		my $chosen = $popsort[ rand( $#popsort )];
-#		push( @offspring, $chosen->clone() );
-#	  }
-#	  my $mutante = $_->apply( @offspring );
-#	  push( @popsort, $mutante );
-#	}
-#  }
 
   #Return
   for ( my $i = 0; $i <= $#popsort; $i++ ) {
@@ -196,15 +176,21 @@ sub apply ($) {
   
 }
 
+=head1 SEE ALSO
+
+L<Algorithm::Evolutionary::Op::CanonicalGA>.
+L<Algorithm::Evolutionary::Op::FullAlgorithm>.
+
+
 =head1 Copyright
   
   This file is released under the GPL. See the LICENSE file included in this distribution,
   or go to http://www.fsf.org/licenses/gpl.txt
 
-  CVS Info: $Date: 2008/06/21 21:05:53 $ 
-  $Header: /cvsroot/opeal/Algorithm-Evolutionary/lib/Algorithm/Evolutionary/Op/Easy.pm,v 1.5 2008/06/21 21:05:53 jmerelo Exp $ 
+  CVS Info: $Date: 2009/02/04 20:43:14 $ 
+  $Header: /cvsroot/opeal/Algorithm-Evolutionary/lib/Algorithm/Evolutionary/Op/Easy.pm,v 2.1 2009/02/04 20:43:14 jmerelo Exp $ 
   $Author: jmerelo $ 
-  $Revision: 1.5 $
+  $Revision: 2.1 $
   $Name $
 
 =cut
